#ifndef __hough
#define __hough

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "lib/iplib.h"

typedef struct
{
  int contthreshold;       // edge map threshold
  int abthreshold;         // (a,b)-space threshold
  int rthreshold;          // r-space threshold (allow for concentric circles)
  int use2ndderivative;    // should the 2nd derivative filter be used?
  int minradius;           // minimum allowed radius of the circles
  int maxradius;           // maximum allowed radius of the circles
  int smoothimage;         // size of smoothing filter
  int saveall;             // should all intermediate files be saved?
  int circlecolour;        // colour to use to draw the circles
  int refinecoordinates;   // should a final refinement of coordinates be used?
  char *inputfilename;
  char *outputfilename;
} TParams;

typedef TParams *PParams;

PParams ProcessParameters(int argc, char* argv[]);

void writefloatpgm (float **im, int w, int h, char *fname, float shift);

int removesmallervalues (float **f, int i, int j, int h, int w);

float **convolution (float **image, int iw, int ih,
                     float **filter, int fw, int fh);

float **smoothimage (float **im, int w, int h, PParams P);

float sobelfilter (float **image, int w, int h,
                   float ***imageh, float ***imagev, float ***imagetot,
                   PParams Params);

float **thresholdandfinddirectionmap (float **sobel, int w, int h,
                  float **sobelh, float **sobelv, float maxval, PParams P);

float **accumulateinab (float **edge, int w, int h, float **dir, PParams P);

float **enhanceabspace (float **abspace, int w, int h, PParams P);

void accumulateinrspace (float **abspace, float **edgemap, float **dirmap,
         int w, int h, unsigned char **image, PParams P);

#else

#endif
